import numpy as np
import random

class RandomPolicy:

  def __init__(self, actions=["left", "right", "up", "down"]):

    self.actions = actions

  def next_action(self, state):
    """Sample uniformly between actions
    Args:
      state : tensor of dimension [height x width x 3]

    Returns:
      an action from 'actions'
    """

    # TODO: implement batch
    action = random.choice(self.actions)

    return action

class SmartRandomPolicy:

  def __init__(self, actions=["left", "right", "up", "down"]):

    self.actions = actions

  def next_action(self, state):
    """Sample uniformly between actions, but avoid actions that lead to certain
    immediate loss, if possible, otherwise sample randomly

    Args:
      state : tensor of dimension [height x width x 3]

    Returns:
      an action from 'actions'
    """

    def find_head():
      """Find current position of agent"""
      height, width = state.shape[-3:-1]
      for i in range(height):
        for j in range(width):
          if state[i, j][1] == 255 and state[i, j][2] == 255:
            return (i, j)

      raise Exception("Could not find pos of player")

    def is_occupied(pos):
      """Unoccupied positions are zero in all coordinates"""

      return np.any(state[pos] != [0, 0, 0])

    def may_work():
      """Return actions that will not necessarily lead to certain immediate loss"""

      pos = find_head()
      move_pos = {
        "left"  : (pos[0], pos[1] - 1),
        "right" : (pos[0], pos[1] + 1),
        "up"    : (pos[0] - 1, pos[1]),
        "down"  : (pos[0] + 1, pos[1])
      }
      options = []
      for move, new_pos in move_pos.items():
        if not is_occupied(new_pos):
          options.append(move)

      return options

    options = may_work()
    # if all actions lead to certain loss, pick any
    if len(options) == 0:
      options = self.actions

    action = random.choice(options)

    return action
